#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import os
from gettext import gettext as _

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gio

from gasimporter.window import ApplicationWindow
from gasimporter.window import ShortcutsWindow

class Application(Gtk.Application):

    def __init__(self):
        Gtk.Application.__init__(self)

        self.window = None

    def do_startup(self):
        Gtk.Application.do_startup(self)

        #self.build_actions()
        self.build_menu()

    def do_activate(self):
        if self.window:
            return
        self.window = ApplicationWindow(self)
        # .show() vs. .show_all() vs. .present()
        # You MUST use .show_all() for headerBar() to work!
        self.window.show_all()

    """
    Build Gio Menu and add acclerator actions.
    """
    def build_menu(self):
        # Init Gio menu (app-menu)
        menu = Gio.Menu()

        # Menu info
        actions = [
            ("preferences", _("Preferences"), self.preferences,["<Control>p", "<Control>comma"], _("Open preferences window.")),
            ("shortcuts", _("Keyboard Shortcuts"), self.shortcuts, ["F2"], _("Open shortcuts window.")),
            ("help", _("Help"), self.help, ["F3"], _("Open help window.")),
            ("about", _("About"), self.about, ["F1"], _("Open about window.")),
            ("quit", _("Quit"), self.quit, ["<Control>q"], _("Quit application.")),
        ]

        for action, title, callback, accelerators, description in actions:
            # Create menu item
            menu.append(title, "app.{}".format(action))
            # Add action to menu item
            simpleAction = Gio.SimpleAction.new(action, None)
            simpleAction.connect('activate', callback)
            self.add_action(simpleAction)
            # Add accelerator action
            if accelerators:
                self.set_accels_for_action("app.{}".format(action), accelerators)
            # Add acclerator to shortcut window
            if description:
                ShortcutsWindow.add_action("app.{}".format(action), description)

        # Add app-menu to app
        self.set_app_menu(menu)

    """
    Callbacks for Gio.Menu
    """
    def preferences(self, action=None, param=None):
        self.window.dialog_preferences()

    def help(self, action=None, param=None):
        self.window.dialog_help()

    def shortcuts(self, action=None, param=None):
        self.window.dialog_shortcuts()

    def about(self, action=None, param=None):
        self.window.dialog_about()

    def quit(self, action=None, param=None):
        self.window.destroy()

if __name__ == "__main__":
    app = Application()
    exit_status = app.run(sys.argv)
    sys.exit(exit_status)
