#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import os
from gettext import gettext as _

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Gio
from gi.repository import GLib

from gasimporter.configuration import APP_NAME
from gasimporter.configuration import APP_URL
from gasimporter.configuration import APP_VERSION
from gasimporter.configuration import APP_DESCRIPTION
from gasimporter.configuration import APP_COMMENTS
from gasimporter.configuration import APP_LICENSE
from gasimporter.configuration import APP_AUTHORS
from gasimporter.configuration import APP_TRANSLATORS
from gasimporter.configuration import APP_DOCUMENTERS
from gasimporter.configuration import APP_ICON_NAME
from gasimporter.configuration import APP_LOGO_NAME
from gasimporter.configuration import APP_MANUAL_URL_OFFLINE

# https://andrewsteele.me.uk/learngtk.org/tutorials/python_gtk3_tutorial/html/application.html

class ApplicationWindow(Gtk.ApplicationWindow):
    def __init__(self, app):
        gtksettings = Gtk.Settings.get_default()
        gtksettings.set_property("gtk-application-prefer-dark-theme", True)
        theme = gtksettings.get_property("gtk-theme-name")
        os.environ["GTK_THEME"] = theme + ":dark"

        #Gtk.Window.__init__(self, title=APP_NAME, application=app)
        Gtk.ApplicationWindow.__init__(self, title=APP_NAME, application=app)
        self.app = app
        self.set_default_size(320, 480)
        self.set_position(Gtk.WindowPosition.CENTER)

        self.build_ui()

    def build_ui(self):
        self.set_icon_name("gas-importer")

        headerbar = Gtk.HeaderBar()
        headerbar.set_title(APP_NAME)
        headerbar.set_show_close_button(True)
        self.set_titlebar(headerbar)

    def dialog_preferences(self):
        preferences = PrefernecesWindow(self)
        preferences.show_all()

    def dialog_help(self):
        Gtk.show_uri(None, APP_MANUAL_URL_OFFLINE, Gdk.CURRENT_TIME)

    def dialog_shortcuts(self):
        shortcut = ShortcutsWindow(self, self.app)
        shortcut.show_all()

    def dialog_about(self):
        about = AboutWindow(self)
        about.show_all()

class AboutWindow(Gtk.AboutDialog):
    def __init__(self, main_window):
        Gtk.AboutDialog.__init__(self)

        self.main_window = main_window
        self.connect("response", self.dialog_response)

        """
        Build Settings
        """
        self.set_modal(True)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.set_transient_for(self.main_window)
        self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)

        """
        Build UI
        """
        self.build_ui()

    def build_ui(self):
        self.set_program_name(APP_NAME)
        self.set_website(APP_URL)
        self.set_version(APP_VERSION)
        self.set_comments(APP_DESCRIPTION)
        self.set_authors(APP_AUTHORS)
        self.set_translator_credits(' '.join(APP_TRANSLATORS))
        self.set_documenters(APP_DOCUMENTERS)
        self.set_license_type(APP_LICENSE)
        self.set_copyright(APP_COMMENTS)
        self.set_icon_name(APP_ICON_NAME)
        self.set_logo_icon_name(APP_LOGO_NAME)

    def dialog_response(self, dialog, response):
        dialog.destroy()


class PrefernecesWindow(Gtk.Window):
    def __init__(self, main_window):
        Gtk.Window.__init__(self)

        self.main_window = main_window
        self.connect('key-press-event', self._keypress)

        """
        Build Settings
        """
        self.set_title(_("Preferneces"))
        self.set_default_size(480, 400)
        self.set_modal(True)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.set_transient_for(self.main_window)
        self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)

        """
        Build UI
        """
        self.build_ui()

    def build_ui(self):
        """
        Build HeaderBar
        """
        headerbar = Gtk.HeaderBar()
        headerbar.set_title(_("Preferneces"))
        headerbar.set_show_close_button(True)
        self.set_titlebar(headerbar)

        """
        Build Content
        """


    def _keypress(self, widget, event):
        if event.keyval == Gdk.KEY_Escape:
            self.destroy()


class ShortcutsWindow(Gtk.ShortcutsWindow):
    group_titles = {}
    group_actions = {}
    groups = []

    def __init__(self, main_window, app):
        Gtk.ShortcutsWindow.__init__(self)

        self.main_window = main_window
        self.app = app

        """
        Build Settings
        """
        self.set_modal(True)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.set_transient_for(self.main_window)
        self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)

        """
        Build UI
        """
        self.populate()

    def populate(self):
        """Gathers the accelerators and populates the window."""
        section = Gtk.ShortcutsSection()
        section.show()
        for group_id in ShortcutsWindow.groups:
            group = Gtk.ShortcutsGroup(title=ShortcutsWindow.group_titles[group_id])
            group.show()
            for action, title in ShortcutsWindow.group_actions[group_id]:
                accelerators = " ".join(self.app.get_accels_for_action(action))
                short = Gtk.ShortcutsShortcut(title=title, accelerator=accelerators)
                short.show()
                group.add(short)
            section.add(group)
        # Method below must be called after the section has been populated,
        # otherwise the shortcuts won't show up in search.
        self.add(section)

    @classmethod
    def add_action(cls, action, title, group=None):
        """Adds an action to be displayed.

        Args:
            action (str): The name identifying the action, formatted like
                "prefix.name".
            title (str): The title of the action.
            group (Optional[str]): The group id registered with `register_group`
                to be used instead of the one extracted from `action`.
        """
        action_prefix = group or action.split(".")[0]
        if action_prefix not in cls.group_actions:
            cls.group_actions[action_prefix] = []
        cls.group_actions[action_prefix].append((action, title))

    @classmethod
    def register_group(cls, action_prefix, title):
        """Registers a group of shortcuts to be displayed.

        Args:
            action_prefix (str): The group id.
            title (str): The title of the group.
        """
        if action_prefix not in cls.groups:
            cls.groups.append(action_prefix)
        cls.group_titles[action_prefix] = title
